# -*- coding: utf-8 -*-

from rest_framework import serializers
from thx.api.serializers import (
    ThuxCreateSerializerMixin, ThuxDisableSerializerMixin, ThuxEnableSerializerMixin, ThuxListSerializerMixin,
    ThuxPartialUpdateSerializerMixin, ThuxRetrieveSerializerMixin, ThuxUpdateSerializerMixin,
)
from thx.role.core.serializers import ThuxRoleSerializerMixin

from ... import models


# BOX
class BoxSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box'
        model = models.Box
        fields = '__all__'


class BoxListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_list'
        model = models.Box
        fields = '__all__'


class BoxRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_retrieve'
        model = models.Box
        fields = '__all__'


class BoxCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_create'
        model = models.Box
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = BoxRetrieveSerializer


class BoxUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_update'
        model = models.Box
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = BoxRetrieveSerializer


class BoxPartialUpdateSerializer(ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_partial_update'
        model = models.Box
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = BoxRetrieveSerializer


class BoxEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_enable'
        model = models.Box
        fields = ()
        response_serializer = BoxRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class BoxDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_disable'
        model = models.Box
        fields = ()
        response_serializer = BoxRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class BoxLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_box_light'
        model = models.Box
        fields = ('id', 'status', 'status_display',)


# TOUR
class TourSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour'
        model = models.Tour
        fields = '__all__'


class TourListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    boxes = BoxListSerializer(many=True, read_only=True)

    class Meta:
        ref_name = 'v1_tour_tour_list'
        model = models.Tour
        fields = '__all__'


class TourRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    boxes = BoxListSerializer(many=True, read_only=True)

    class Meta:
        ref_name = 'v1_tour_tour_retrieve'
        model = models.Tour
        fields = '__all__'


class TourGetByCodeSerializer(TourRetrieveSerializer):
    class Meta(TourRetrieveSerializer.Meta):
        ref_name = 'v1_tour_tour_get_by_code'


class TourCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour_create'
        model = models.Tour
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = TourRetrieveSerializer


class TourUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour_update'
        model = models.Tour
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = TourRetrieveSerializer


class TourPartialUpdateSerializer(ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour_partial_update'
        model = models.Tour
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = TourRetrieveSerializer


class TourEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour_enable'
        model = models.Tour
        fields = ()
        response_serializer = TourRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class TourDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour_disable'
        model = models.Tour
        fields = ()
        response_serializer = TourRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class TourLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_tour_tour_light'
        model = models.Tour
        fields = ('id', 'status', 'status_display',)
